@@ -27,7 +27,6 @@
 #include "PolySetCache.h"
 #include "ModuleCache.h"
 #include "MainWindow.h"
-#include "openscad.h" // examplesdir
 #include "parsersettings.h"
 #include "Preferences.h"
 #include "printutils.h"
@@ -76,6 +75,7 @@
 #include <QSettings>
 #include <QProgressDialog>
 #include <QMutexLocker>
+#include <QTemporaryFile>
 
 #include <fstream>
 
@@ -101,16 +101,14 @@
 
 #endif // ENABLE_CGAL
 
-#ifndef OPENCSG_VERSION_STRING
-#define OPENCSG_VERSION_STRING "unknown, <1.3.2"
-#endif
-
 #include "boosty.h"
 
-extern QString examplesdir;
+// Keeps track of open window
+QSet<MainWindow*> *MainWindow::windows = NULL;
 
 // Global application state
 unsigned int GuiLocker::gui_locked = 0;
+QString MainWindow::qexamplesdir;
 
 #define QUOTE(x__) # x__
 #define QUOTED(x__) QUOTE(x__)
@@ -122,7 +120,7 @@ static char helptitle[] =
 #endif
 	"\nhttp://www.openscad.org\n\n";
 static char copyrighttext[] =
-	"Copyright (C) 2009-2013 The OpenSCAD Developers\n"
+	"Copyright (C) 2009-2014 The OpenSCAD Developers\n"
 	"\n"
 	"This program is free software; you can redistribute it and/or modify "
 	"it under the terms of the GNU General Public License as published by "
@@ -161,9 +159,15 @@ settings_valueList(const QString &key, const QList<int> &defaultList = QList<int
 }
 
 MainWindow::MainWindow(const QString &filename)
-	: root_inst("group"), progresswidget(NULL)
+	: root_inst("group"), tempFile(NULL), progresswidget(NULL)
 {
 	setupUi(this);
+  // FIXME: We cannot do this since Context maintains a global stack which gets pushed/popped when
+  // mainwindows are created. To fix, we probably need a separate stack per window. kintel 20140309
+//	this->setAttribute(Qt::WA_DeleteOnClose);
+
+	if (!MainWindow::windows) MainWindow::windows = new QSet<MainWindow*>;
+	MainWindow::windows->insert(this);
 
 #ifdef ENABLE_CGAL
 	this->cgalworker = new CGALWorker();
@@ -194,7 +198,8 @@ MainWindow::MainWindow(const QString &filename)
 	fps = 0;
 	fsteps = 1;
 
-	highlighter = new Highlighter(editor->document());
+	connect(this, SIGNAL(highlightError(int)), editor, SLOT(highlightError(int)));
+	connect(this, SIGNAL(unhighlightLastError()), editor, SLOT(unhighlightLastError()));
 	editor->setTabStopWidth(30);
 	editor->setLineWrapping(true); // Not designable
 
@@ -206,12 +211,19 @@ MainWindow::MainWindow(const QString &filename)
 
 	autoReloadTimer = new QTimer(this);
 	autoReloadTimer->setSingleShot(false);
+	autoReloadTimer->setInterval(200);
 	connect(autoReloadTimer, SIGNAL(timeout()), this, SLOT(checkAutoReload()));
 
-	connect(this->e_tval, SIGNAL(textChanged(QString)), this, SLOT(actionCompile()));
+	waitAfterReloadTimer = new QTimer(this);
+	waitAfterReloadTimer->setSingleShot(true);
+	waitAfterReloadTimer->setInterval(200);
+	connect(waitAfterReloadTimer, SIGNAL(timeout()), this, SLOT(waitAfterReload()));
+
+	connect(this->e_tval, SIGNAL(textChanged(QString)), this, SLOT(actionRenderCSG()));
 	connect(this->e_fps, SIGNAL(textChanged(QString)), this, SLOT(updatedFps()));
 
 	animate_panel->hide();
+	find_panel->hide();
 
 	// Application menu
 #ifdef DEBUG
@@ -251,10 +263,9 @@ MainWindow::MainWindow(const QString &filename)
 	this->menuOpenRecent->addAction(this->fileActionClearRecent);
 	connect(this->fileActionClearRecent, SIGNAL(triggered()),
 					this, SLOT(clearRecentFiles()));
-
-	if (!examplesdir.isEmpty()) {
+	if (!qexamplesdir.isEmpty()) {
 		bool found_example = false;
-		QStringList examples = QDir(examplesdir).entryList(QStringList("*.scad"), 
+		QStringList examples = QDir(qexamplesdir).entryList(QStringList("*.scad"), 
 		QDir::Files | QDir::Readable, QDir::Name);
 		foreach (const QString &ex, examples) {
 			this->menuExamples->addAction(ex, this, SLOT(actionOpenExample()));
@@ -285,16 +296,23 @@ MainWindow::MainWindow(const QString &filename)
 	connect(this->editActionZoomOut, SIGNAL(triggered()), editor, SLOT(zoomOut()));
 	connect(this->editActionHide, SIGNAL(triggered()), this, SLOT(hideEditor()));
 	connect(this->editActionPreferences, SIGNAL(triggered()), this, SLOT(preferences()));
+	// Edit->Find
+	connect(this->editActionFind, SIGNAL(triggered()), this, SLOT(find()));
+	connect(this->editActionFindAndReplace, SIGNAL(triggered()), this, SLOT(findAndReplace()));
+	connect(this->editActionFindNext, SIGNAL(triggered()), this, SLOT(findNext()));
+	connect(this->editActionFindPrevious, SIGNAL(triggered()), this, SLOT(findPrev()));
+	connect(this->editActionUseSelectionForFind, SIGNAL(triggered()), this, SLOT(useSelectionForFind()));
 
 	// Design menu
 	connect(this->designActionAutoReload, SIGNAL(toggled(bool)), this, SLOT(autoReloadSet(bool)));
-	connect(this->designActionReloadAndCompile, SIGNAL(triggered()), this, SLOT(actionReloadCompile()));
-	connect(this->designActionCompile, SIGNAL(triggered()), this, SLOT(actionCompile()));
+	connect(this->designActionReloadAndCompile, SIGNAL(triggered()), this, SLOT(actionReloadRenderCSG()));
+	connect(this->designActionCompile, SIGNAL(triggered()), this, SLOT(actionRenderCSG()));
 #ifdef ENABLE_CGAL
 	connect(this->designActionCompileAndRender, SIGNAL(triggered()), this, SLOT(actionRenderCGAL()));
 #else
 	this->designActionCompileAndRender->setVisible(false);
 #endif
+	connect(this->designCheckValidity, SIGNAL(triggered()), this, SLOT(actionCheckValidity()));
 	connect(this->designActionDisplayAST, SIGNAL(triggered()), this, SLOT(actionDisplayAST()));
 	connect(this->designActionDisplayCSGTree, SIGNAL(triggered()), this, SLOT(actionDisplayCSGTree()));
 	connect(this->designActionDisplayCSGProducts, SIGNAL(triggered()), this, SLOT(actionDisplayCSGProducts()));
@@ -335,6 +353,7 @@ MainWindow::MainWindow(const QString &filename)
 	connect(this->viewActionBack, SIGNAL(triggered()), this, SLOT(viewAngleBack()));
 	connect(this->viewActionDiagonal, SIGNAL(triggered()), this, SLOT(viewAngleDiagonal()));
 	connect(this->viewActionCenter, SIGNAL(triggered()), this, SLOT(viewCenter()));
+	connect(this->viewActionResetView, SIGNAL(triggered()), this, SLOT(viewResetView()));
 	connect(this->viewActionPerspective, SIGNAL(triggered()), this, SLOT(viewPerspective()));
 	connect(this->viewActionOrthogonal, SIGNAL(triggered()), this, SLOT(viewOrthogonal()));
 	connect(this->viewActionHide, SIGNAL(triggered()), this, SLOT(hideConsole()));
@@ -369,8 +388,21 @@ MainWindow::MainWindow(const QString &filename)
 					this, SLOT(setFont(const QString&,uint)));
 	connect(Preferences::inst(), SIGNAL(openCSGSettingsChanged()),
 					this, SLOT(openCSGSettingsChanged()));
+	connect(Preferences::inst(), SIGNAL(syntaxHighlightChanged(const QString&)), 
+					editor, SLOT(setHighlightScheme(const QString&)));
 	Preferences::inst()->apply();
 
+	connect(this->findTypeComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(selectFindType(int)));
+	connect(this->findInputField, SIGNAL(returnPressed()), this->nextButton, SLOT(animateClick()));
+	find_panel->installEventFilter(this);
+
+	connect(this->prevButton, SIGNAL(clicked()), this, SLOT(findPrev()));
+	connect(this->nextButton, SIGNAL(clicked()), this, SLOT(findNext()));
+	connect(this->hideFindButton, SIGNAL(clicked()), find_panel, SLOT(hide()));
+	connect(this->replaceButton, SIGNAL(clicked()), this, SLOT(replace()));
+	connect(this->replaceAllButton, SIGNAL(clicked()), this, SLOT(replaceAll()));
+	connect(this->replaceInputField, SIGNAL(returnPressed()), this->replaceButton, SLOT(animateClick()));
+
 	// make sure it looks nice..
 	QSettings settings;
 	resize(settings.value("window/size", QSize(800, 600)).toSize());
@@ -452,6 +484,7 @@ MainWindow::~MainWindow()
 #ifdef ENABLE_OPENCSG
 	delete this->opencsgRenderer;
 #endif
+	MainWindow::windows->remove(this);
 }
 
 void MainWindow::showProgress()
@@ -504,6 +537,8 @@ MainWindow::openFile(const QString &new_filename)
 	}
 #endif
 	setFileName(actual_filename);
+	editor->setPlainText("");
+	this->last_compiled_doc = "";
 
 	fileChangedOnDisk(); // force cached autoReloadId to update
 	refreshDocument();
@@ -580,7 +615,7 @@ void MainWindow::updateTVal()
 	double fps = this->e_fps->text().toDouble(&fps_ok);
 	if (fps_ok) {
 		if (fps <= 0) {
-			actionCompile();
+			actionRenderCSG();
 		} else {
 			double s = this->e_fsteps->text().toDouble();
 			double t = this->e_tval->text().toDouble() + 1/s;
@@ -612,14 +647,109 @@ void MainWindow::refreshDocument()
 }
 
 /*!
-	Parse and evaluate the design => this->root_node
-
-	Returns true if something was compiled, false if nothing was changed
-  and the root_node was left untouched.
+	compiles the design. Calls compileDone() if anything was compiled
 */
-bool MainWindow::compile(bool reload, bool procevents)
+void MainWindow::compile(bool reload, bool forcedone)
 {
-	if (!compileTopLevelDocument(reload)) return false;
+	bool shouldcompiletoplevel = false;
+	bool didcompile = false;
+
+	// Reload checks the timestamp of the toplevel file and refreshes if necessary,
+	if (reload) {
+		// Refresh files if it has changed on disk
+		if (fileChangedOnDisk() && checkEditorModified()) {
+			shouldcompiletoplevel = true;
+			refreshDocument();
+		}
+		// If the file hasn't changed, we might still need to compile it
+		// if we haven't yet compiled the current text.
+		else {
+			QString current_doc = editor->toPlainText();
+			if (current_doc != last_compiled_doc && last_compiled_doc.size() == 0) {
+				shouldcompiletoplevel = true;
+			}
+		}
+	}
+	else {
+		shouldcompiletoplevel = true;
+	}
+
+	if (!shouldcompiletoplevel && this->root_module && this->root_module->includesChanged()) {
+		shouldcompiletoplevel = true;
+	}
+
+	if (shouldcompiletoplevel) {
+		console->clear();
+		if (editor->isContentModified()) saveBackup();
+		compileTopLevelDocument();
+		didcompile = true;
+	}
+
+	if (this->root_module) {
+		if (this->root_module->handleDependencies()) {
+			PRINTB("Module cache size: %d modules", ModuleCache::instance()->size());
+			didcompile = true;
+		}
+	}
+
+	// If we're auto-reloading, listen for a cascade of changes by starting a timer
+	// if something changed _and_ there are any external dependencies
+	if (reload && didcompile && this->root_module) {
+		if (this->root_module->hasIncludes() ||
+				this->root_module->usesLibraries()) {
+			this->waitAfterReloadTimer->start();
+			return;
+		}
+	}
+
+	if (!reload && didcompile) {
+		if (!animate_panel->isVisible()) {
+			emit unhighlightLastError();
+			if (!this->root_module) {
+				emit highlightError( parser_error_pos );
+			}
+		}
+	}
+
+	compileDone(didcompile | forcedone);
+}
+
+void MainWindow::waitAfterReload()
+{
+	if (this->root_module->handleDependencies()) {
+		this->waitAfterReloadTimer->start();
+		return;
+	}
+	else {
+		compile(true, true); // In case file itself or top-level includes changed during dependency updates
+	}
+}
+
+void MainWindow::compileDone(bool didchange)
+{
+	const char *callslot;
+	if (didchange) {
+		instantiateRoot();
+		callslot = afterCompileSlot;
+	}
+	else {
+		callslot = "compileEnded";
+	}
+
+	this->procevents = false;
+	QMetaObject::invokeMethod(this, callslot);
+}
+
+void MainWindow::compileEnded()
+{
+	clearCurrentOutput();
+	GuiLocker::unlock();
+	if (designActionAutoReload->isChecked()) autoReloadTimer->start();
+}
+
+void MainWindow::instantiateRoot()
+{
+	// Go on and instantiate root_node, then call the continuation slot
 
   // Invalidate renderers before we kill the CSG tree
 	this->qglview->setRenderer(NULL);
@@ -652,7 +782,7 @@ bool MainWindow::compile(bool reload, bool procevents)
 	if (this->root_module) {
 		// Evaluate CSG tree
 		PRINT("Compiling design (CSG Tree generation)...");
-		if (procevents) QApplication::processEvents();
+		if (this->procevents) QApplication::processEvents();
 		
 		AbstractNode::resetIndexCounter();
 
@@ -681,10 +811,8 @@ bool MainWindow::compile(bool reload, bool procevents)
 		} else {
 			PRINT("ERROR: Compilation failed!");
 		}
-		if (procevents) QApplication::processEvents();
+		if (this->procevents) QApplication::processEvents();
 	}
-
-	return true;
 }
 
 /*!
@@ -901,10 +1029,49 @@ void MainWindow::actionOpenExample()
 {
 	QAction *action = qobject_cast<QAction *>(sender());
 	if (action) {
-		openFile(examplesdir + QDir::separator() + action->text());
+		openFile(qexamplesdir + QDir::separator() + action->text());
 	}
 }
 
+void MainWindow::writeBackup(QFile *file)
+{
+	// see MainWindow::saveBackup()
+	file->resize(0);
+	QTextStream writer(file);
+	writer.setCodec("UTF-8");
+	writer << this->editor->toPlainText();
+	
+	PRINTB("Saved backup file: %s", file->fileName().toLocal8Bit().constData());
+}
+
+void MainWindow::saveBackup()
+{
+	std::string path = PlatformUtils::backupPath();
+	if ((!fs::exists(path)) && (!PlatformUtils::createBackupPath())) {
+		PRINTB("WARNING: Cannot create backup path: %s", path);
+		return;
+	}
+
+	QString backupPath = QString::fromStdString(path);
+	if (!backupPath.endsWith("/")) backupPath.append("/");
+
+	QString basename = "unsaved";
+	if (!this->fileName.isEmpty()) {
+		QFileInfo fileInfo = QFileInfo(this->fileName);
+		basename = fileInfo.baseName();
+	}
+
+	if (!this->tempFile) {
+		this->tempFile = new QTemporaryFile(backupPath.append(basename + "-backup-XXXXXXXX.scad"));
+	}
+
+	if ((!this->tempFile->isOpen()) && (! this->tempFile->open())) {
+		PRINT("WARNING: Failed to create backup file");
+		return;
+	}
+	return writeBackup(this->tempFile);
+}
+
 void MainWindow::actionSave()
 {
 	if (this->fileName.isEmpty()) {
@@ -981,10 +1148,10 @@ void MainWindow::hideEditor()
 {
 	QSettings settings;
 	if (editActionHide->isChecked()) {
-		editor->hide();
+		editorPane->hide();
 		settings.setValue("view/hideEditor",true);
 	} else {
-		editor->show();
+		editorPane->show();
 		settings.setValue("view/hideEditor",false);
 	}
 }
@@ -1006,6 +1173,102 @@ void MainWindow::pasteViewportRotation()
 	cursor.insertText(txt);
 }
 
+void MainWindow::find()
+{
+	findTypeComboBox->setCurrentIndex(0);
+	replaceInputField->hide();
+	replaceButton->hide();
+	replaceAllButton->hide();
+	find_panel->show();
+	findInputField->setFocus();
+	findInputField->selectAll();
+}
+
+void MainWindow::findAndReplace()
+{
+	findTypeComboBox->setCurrentIndex(1);
+	replaceInputField->show();
+	replaceButton->show();
+	replaceAllButton->show();
+	find_panel->show();
+	findInputField->setFocus();
+	findInputField->selectAll();
+}
+
+void MainWindow::selectFindType(int type) {
+	if (type == 0) find();
+	if (type == 1) findAndReplace();
+}
+
+bool MainWindow::findOperation(QTextDocument::FindFlags options) {
+	bool success = editor->find(findInputField->text(), options);
+	if (!success) { // Implement wrap-around search behavior
+		QTextCursor old_cursor = editor->textCursor();
+		QTextCursor tmp_cursor = old_cursor;
+		tmp_cursor.movePosition((options & QTextDocument::FindBackward) ? QTextCursor::End : QTextCursor::Start);
+		editor->setTextCursor(tmp_cursor);
+		bool success = editor->find(findInputField->text(), options);
+		if (!success) {
+			editor->setTextCursor(old_cursor);
+		}
+		return success;
+	}
+	return true;
+}
+
+void MainWindow::replace() {
+	QTextCursor cursor = editor->textCursor();
+	QString selectedText = cursor.selectedText();
+	if (selectedText == findInputField->text()) {
+		cursor.insertText(replaceInputField->text());
+	}
+	findNext();
+}
+
+void MainWindow::replaceAll() {
+	QTextCursor old_cursor = editor->textCursor();
+	QTextCursor tmp_cursor = old_cursor;
+	tmp_cursor.movePosition(QTextCursor::Start);
+	editor->setTextCursor(tmp_cursor);
+	while (editor->find(findInputField->text())) {
+		editor->textCursor().insertText(replaceInputField->text());
+	}
+	editor->setTextCursor(old_cursor);
+}
+
+void MainWindow::findNext()
+{
+	findOperation();
+}
+
+void MainWindow::findPrev()
+{
+	findOperation(QTextDocument::FindBackward);
+}
+
+void MainWindow::useSelectionForFind()
+{
+	findInputField->setText(editor->textCursor().selectedText());
+}
+
+bool MainWindow::eventFilter(QObject* obj, QEvent *event)
+{
+    if (obj == find_panel)
+    {
+        if (event->type() == QEvent::KeyPress)
+        {
+            QKeyEvent* keyEvent = static_cast<QKeyEvent*>(event);
+            if (keyEvent->key() == Qt::Key_Escape)
+            {
+				find_panel->hide();
+				return true;
+            }
+        }
+        return false;
+    }
+    return QMainWindow::eventFilter(obj, event);
+}
+
 void MainWindow::updateTemporalVariables()
 {
 	this->top_ctx.set_variable("$t", Value(this->e_tval->text().toDouble()));
@@ -1023,6 +1286,10 @@ void MainWindow::updateTemporalVariables()
 	top_ctx.set_variable("$vpr", Value(vpr));
 }
 
+/*!
+	Returns true if the current document is a file on disk and that file has new content.
+	Returns false if a file on disk has disappeared or if we haven't yet saved.
+*/
 bool MainWindow::fileChangedOnDisk()
 {
 	if (!this->fileName.isEmpty()) {
@@ -1043,77 +1310,34 @@ bool MainWindow::fileChangedOnDisk()
 }
 
 /*!
-	If reload is true, does a timestamp check on the document and tries to reload it.
-	Otherwise, just reparses the current document and any dependencies, updates the 
-	GUI accordingly and populates this->root_module.
-
 	Returns true if anything was compiled.
 */
-bool MainWindow::compileTopLevelDocument(bool reload)
+void MainWindow::compileTopLevelDocument()
 {
-	bool shouldcompiletoplevel = !reload;
-
-	if (this->root_module && this->root_module->includesChanged()) {
-		shouldcompiletoplevel = true;
-	}
-
-	if (reload) {	
-		// Refresh file if it has changed on disk
-	  if (fileChangedOnDisk() && checkEditorModified()) {
-			shouldcompiletoplevel = true;
-			refreshDocument();
-		}
-	  // If the file hasn't changed, we might still need to compile it
-	  // if we haven't yet compiled the current text.
-		else if (!editor->isContentModified()) {
-			QString current_doc = editor->toPlainText();
-			if (current_doc != last_compiled_doc) shouldcompiletoplevel = true;
-		}
-	}
+	updateTemporalVariables();
+	resetPrintedDeprecations();
+	
+	this->last_compiled_doc = editor->toPlainText();
+	std::string fulltext = 
+		std::string(this->last_compiled_doc.toLocal8Bit().constData()) +
+		"\n" + commandline_commands;
+	
+	delete this->root_module;
+	this->root_module = NULL;
+	
+	this->root_module = parse(fulltext.c_str(),
+														this->fileName.isEmpty() ? 
+														"" : 
+														QFileInfo(this->fileName).absolutePath().toLocal8Bit(), 
+														false);
 	
-	if (shouldcompiletoplevel) {
-		console->clear();
-
-		updateTemporalVariables();
-		
-		this->last_compiled_doc = editor->toPlainText();
-		std::string fulltext = 
-			std::string(this->last_compiled_doc.toLocal8Bit().constData()) +
-			"\n" + commandline_commands;
-		
-		delete this->root_module;
-		this->root_module = NULL;
-		
-		this->root_module = parse(fulltext.c_str(),
-															this->fileName.isEmpty() ? 
-															"" : 
-															QFileInfo(this->fileName).absolutePath().toLocal8Bit(), 
-															false);
-
-		if (!animate_panel->isVisible()) {
-			highlighter->unhighlightLastError();
-			if (!this->root_module) {
-				QTextCursor cursor = editor->textCursor();
-				cursor.setPosition(parser_error_pos);
-				editor->setTextCursor(cursor);
-				highlighter->highlightError( parser_error_pos );
-			}
-		}
-
-	}
-
-	bool changed = shouldcompiletoplevel;
-	if (this->root_module) {
-		changed |= this->root_module->handleDependencies();
-		if (changed) PRINTB("Module cache size: %d modules", ModuleCache::instance()->size());
-	}
-
-	return changed;
 }
 
 void MainWindow::checkAutoReload()
 {
-	if (!this->fileName.isEmpty()) actionReloadCompile();
+	if (!this->fileName.isEmpty()) {
+		actionReloadRenderCSG();
+	}
 }
 
 void MainWindow::autoReloadSet(bool on)
@@ -1143,15 +1367,22 @@ bool MainWindow::checkEditorModified()
 	return true;
 }
 
-void MainWindow::actionReloadCompile()
+void MainWindow::actionReloadRenderCSG()
 {
 	if (GuiLocker::isLocked()) return;
-	GuiLocker lock;
+	GuiLocker::lock();
+	autoReloadTimer->stop();
 	setCurrentOutput();
 
 	// PRINT("Parsing design (AST generation)...");
 	// QApplication::processEvents();
-	if (!compile(true, true)) return;
+	this->afterCompileSlot = "csgReloadRender";
+	this->procevents = true;
+	compile(true);
+}
+
+void MainWindow::csgReloadRender()
+{
 	if (this->root_node) compileCSG(true);
 
 	// Go to non-CGAL view mode
@@ -1165,20 +1396,25 @@ void MainWindow::actionReloadCompile()
 		viewModeThrownTogether();
 #endif
 	}
-
-	clearCurrentOutput();
+	compileEnded();
 }
 
-void MainWindow::actionCompile()
+void MainWindow::actionRenderCSG()
 {
 	if (GuiLocker::isLocked()) return;
-	GuiLocker lock;
+	GuiLocker::lock();
+	autoReloadTimer->stop();
 	setCurrentOutput();
-	console->clear();
 
 	PRINT("Parsing design (AST generation)...");
 	QApplication::processEvents();
-	compile(false, !viewActionAnimate->isChecked());
+	this->afterCompileSlot = "csgRender";
+	this->procevents = !viewActionAnimate->isChecked();
+	compile(false);
+}
+
+void MainWindow::csgRender()
+{
 	if (this->root_node) compileCSG(!viewActionAnimate->isChecked());
 
 	// Go to non-CGAL view mode
@@ -1202,7 +1438,7 @@ void MainWindow::actionCompile()
 		img.save(filename, "PNG");
 	}
 	
-	clearCurrentOutput();
+	compileEnded();
 }
 
 #ifdef ENABLE_CGAL
@@ -1210,16 +1446,21 @@ void MainWindow::actionCompile()
 void MainWindow::actionRenderCGAL()
 {
 	if (GuiLocker::isLocked()) return;
-	GuiLocker lock;
-
+	GuiLocker::lock();
+	autoReloadTimer->stop();
 	setCurrentOutput();
-	console->clear();
 
 	PRINT("Parsing design (AST generation)...");
 	QApplication::processEvents();
-	compile(false, true);
+	this->afterCompileSlot = "cgalRender";
+	this->procevents = true;
+	compile(false);
+}
 
+void MainWindow::cgalRender()
+{
 	if (!this->root_module || !this->root_node) {
+        compileEnded();
 		return;
 	}
 
@@ -1238,7 +1479,6 @@ void MainWindow::actionRenderCGAL()
 
 	progress_report_prep(this->root_node, report_func, this);
 
-	GuiLocker::lock(); // Will be unlocked in actionRenderCGALDone()
 	this->cgalworker->start(this->tree);
 }
 
@@ -1267,7 +1507,6 @@ void MainWindow::actionRenderCGALDone(CGAL_Nef_polyhedron *root_N)
 			if (root_N->dim == 3) {
 				PRINT("   Top level object is a 3D object:");
 				PRINTB("   Simple:     %6s", (root_N->p3->is_simple() ? "yes" : "no"));
-				PRINTB("   Valid:      %6s", (root_N->p3->is_valid() ? "yes" : "no"));
 				PRINTB("   Vertices:   %6d", root_N->p3->number_of_vertices());
 				PRINTB("   Halfedges:  %6d", root_N->p3->number_of_halfedges());
 				PRINTB("   Edges:      %6d", root_N->p3->number_of_edges());
@@ -1301,9 +1540,7 @@ void MainWindow::actionRenderCGALDone(CGAL_Nef_polyhedron *root_N)
 	this->statusBar()->removeWidget(this->progresswidget);
 	delete this->progresswidget;
 	this->progresswidget = NULL;
-	clearCurrentOutput();
-
-	GuiLocker::unlock();
+	compileEnded();
 }
 
 #endif /* ENABLE_CGAL */
@@ -1363,6 +1600,30 @@ void MainWindow::actionDisplayCSGProducts()
 	clearCurrentOutput();
 }
 
+void MainWindow::actionCheckValidity() {
+	if (GuiLocker::isLocked()) return;
+	GuiLocker lock;
+#ifdef ENABLE_CGAL
+	setCurrentOutput();
+
+	if (!this->root_N) {
+		PRINT("Nothing to validate! Try building first (press F6).");
+		clearCurrentOutput();
+		return;
+	}
+
+	if (this->root_N->dim != 3) {
+		PRINT("Current top level object is not a 3D object.");
+		clearCurrentOutput();
+		return;
+	}
+
+	bool valid = this->root_N->p3->is_valid();
+	PRINTB("   Valid:      %6s", (valid ? "yes" : "no"));
+	clearCurrentOutput();
+#endif /* ENABLE_CGAL */
+}
+
 #ifdef ENABLE_CGAL
 void MainWindow::actionExportSTLorOFF(bool stl_mode)
 #else
@@ -1616,7 +1877,7 @@ void MainWindow::viewModeAnimate()
 {
 	if (viewActionAnimate->isChecked()) {
 		animate_panel->show();
-		actionCompile();
+		actionRenderCSG();
 		updatedFps();
 	} else {
 		animate_panel->hide();
@@ -1713,6 +1974,12 @@ void MainWindow::viewOrthogonal()
 	this->qglview->updateGL();
 }
 
+void MainWindow::viewResetView()
+{
+	this->qglview->resetView();
+	this->qglview->updateGL();
+}
+
 void MainWindow::hideConsole()
 {
 	QSettings settings;
@@ -1764,34 +2031,16 @@ MainWindow::helpManual()
 	QDesktopServices::openUrl(QUrl("http://www.openscad.org/documentation.html"));
 }
 
-#define STRINGIFY(x) #x
-#define TOSTRING(x) STRINGIFY(x)
 void MainWindow::helpLibrary()
 {
-	QString libinfo;
-	libinfo.sprintf("Boost version: %s\n"
-									"Eigen version: %d.%d.%d\n"
-									"CGAL version: %s\n"
-									"OpenCSG version: %s\n"
-									"Qt version: %s\n\n",
-									BOOST_LIB_VERSION,
-									EIGEN_WORLD_VERSION, EIGEN_MAJOR_VERSION, EIGEN_MINOR_VERSION,
-									TOSTRING(CGAL_VERSION),
-									OPENCSG_VERSION_STRING,
-									qVersion());
-
-#if defined( __MINGW64__ )
-	libinfo += QString("Compiled for MingW64\n\n");
-#elif defined( __MINGW32__ )
-	libinfo += QString("Compiled for MingW32\n\n");
-#endif
-
+	QString info( PlatformUtils::info().c_str() );
+	info += QString( qglview->getRendererInfo().c_str() );
 	if (!this->openglbox) {
-    this->openglbox = new QMessageBox(QMessageBox::Information, 
-                                      "OpenGL Info", "OpenSCAD Detailed Library Info                  ",
+		this->openglbox = new QMessageBox(QMessageBox::Information,
+                                      "OpenGL Info", "OpenSCAD Detailed Library and Build Information",
                                       QMessageBox::Ok, this);
 	}
-  this->openglbox->setDetailedText(libinfo + QString(qglview->getRendererInfo().c_str()));
+	this->openglbox->setDetailedText( info );
 	this->openglbox->show();
 }
 
@@ -1826,6 +2075,10 @@ void MainWindow::closeEvent(QCloseEvent *event)
 		settings.setValue("window/position", pos());
 		settings_setValueList("window/splitter1sizes",splitter1->sizes());
 		settings_setValueList("window/splitter2sizes",splitter2->sizes());
+		if (this->tempFile) {
+			delete this->tempFile;
+			this->tempFile = NULL;
+		}
 		event->accept();
 	} else {
 		event->ignore();
